# NumericAnnex

NumericAnnex supplements the numeric facilities provided in the Swift standard
library.


## Features

- [x] `Math`, a protocol for types providing square root, cube root, and
      elementary transcendental functions.
- [x] `FloatingPointMath`, a protocol for floating point types providing a
      selection of special functions such as the gamma function.
- [x] `Complex`, a value type to represent complex values in Cartesian form,
      which provides all functions required by `Math`.
- [x] `Rational`, a value type to represent rational values, which supports the
      special values infinity, negative infinity, and NaN ("not a number").

> Note: This project is in the early stages of development and is not
> production-ready at this time.


## Requirements

NumericAnnex now requires the latest development (master) branch of Swift or a
recent development snapshot that includes the revised numeric protocols. It
requires either `Darwin` or `Glibc` for transcendental functions provided by the
C standard library.


## Installation

After NumericAnnex has been cloned or downloaded locally, build the library by
invoking `swift build`, or run tests with `swift test`. An Xcode project can be
generated by invoking `swift package generate-xcodeproj`.

[Swift Package Manager](https://swift.org/package-manager/) can also be used to
add the package as a dependency for your own project. See Swift documentation
for details.


## Documentation

All public protocols, types, and functions have been carefully documented in
comments.

The project adheres to many design patterns found in the Swift standard library.
For example, `Math` types provide methods such as `cubeRoot()` and `tangent()`
just as `FloatingPoint` types provide methods such as `squareRoot()`.

No free functions are declared in this library unless they overload existing
ones in the Swift standard library. Instead, functions such as `cbrt(_:)` and
`tan(_:)` are provided as static members. This avoids collisions with C standard
library functions that you may wish to use. It also promotes clarity at the use
site that can be invaluable, for example, when the result of a complex operation
differs from that of its real counterpart (`Complex128.cbrt(-8) != -2`). If the
type of an expression can be unambiguously inferred, you can use a terser
notation, if desired (`_ = 2.5 + .cbrt(3)`).


## Future directions

- [ ] Add more tests
- [ ] Design and implement `BigInt`
- [ ] Design and implement `Random`


## License

All original work is released under the MIT license. See
[LICENSE](https://github.com/xwu/NumericAnnex/blob/master/LICENSE) for details.

Portions of the complex square root and elementary transcendental functions use
checks for special values adapted from libc++. Code in libc++ is dual-licensed
under the MIT and UIUC/NCSA licenses.
